#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import argparse
import pickle
import logging

def orthologs_transfer(manual_annotation_file, orthofinder_file, gff_file, outdir):
    """
    Transfer of description between a manual annotation file and a gff file, using an orthofinder output file
    to find the corresponding IDs between the two files

    :param manual_annotation_file:
    :param orthofinder_file:
    :param gff_file:
    :param outdir:
    :return:
    """

    manual_annotations_dict, orthofinder_dict, gff_dict = {}, {}, {}
    species_filename = ""

    with open(orthofinder_file, 'r') as orthofinder:
        next(orthofinder)  # Skip header
        # Mapping orthofinder's IDs to manual annotation's description  
        logging.info("Mapping Orthofinder's IDs to manual annotation's IDs...")
        for orthofiner_line in orthofinder:
            orthofiner_line_split = orthofiner_line.rstrip().split("\t")
            orthofinder_dict[orthofiner_line_split[1].split(" ")[0]] = orthofiner_line_split[2].split(" ")[0]
    

        with open(manual_annotation_file, 'r') as manual_annotation:
            # Mapping orthofinder's IDs to manual annotation's description
            logging.info("Mapping Orthofinder's IDs to descriptions...")
            next(manual_annotation)  # Skip header
            for manual_annotation_line in manual_annotation:
                manual_annotation_line_split = manual_annotation_line.rstrip().split("\t")
                manual_annotations_dict[manual_annotation_line_split[0]] = ";".join([manual_annotation_line_split[0], manual_annotation_line_split[6]])

    # Opening GFF, appending manual annotation's description to matching IDs
    logging.info("Transferring manual descriptions to the GFF file...")
    output_filename = str(os.path.join(os.path.abspath(outdir), os.path.basename(gff_file).split(".")[0]) + "_TRANSFERED.gff")

    with open(output_filename, 'a') as output_file:
        output_file.truncate(0)  # Erase previous file content
        with open(gff_file, 'r') as gff:
            for gff_line in gff:
                if "ID=mRNA" in gff_line:  # Look for mRNA items
                    gff_line_id = gff_line.split("\t")[8].split("=")[1].split(";")[0]  # GFF file ID (matches manual annotation IDs)
                    if gff_line_id in orthofinder_dict.keys():  # The gff ID is supposed to match a unique ID (key) in the orthofinder file
                        try:
                            manual_annotation_value = orthofinder_dict[gff_line_id]  # Find the corresponding manual annotation to transfer
                            gff_line = "{0};ec32_ortholog={1};ec32_ortholog_description={2}\n".format(gff_line.strip(), orthofinder_dict[gff_line_id], manual_annotation_value)
                        except KeyError:  # Just in case some values are missing in the manual annotation file (optional, can be removed, it will then exit instead)
                            continue
                output_file.write(gff_line)
            logging.info("Finished transferring descriptions for %s" % str(os.path.basename(gff_file)))


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description="Orthologs_transfer between a reference manual annotation and Orthofinder outputs. \
                                                  \n\nRun example: python3 ec32_orthologs_transfer.py -d /path/to/desired/dir/ -o orthofinder_file.tsv \
                                                  -g gff_file.gff -m manual_annotation_file.txt")
    parser.add_argument("-o", "--orthofinder",
                        help="Orthofinder output (tabulated)")
    parser.add_argument("-m", "--manual-annotation",
                        help="Manual annotation file (tabulated) or pickle file containing the dicionary (automatically generated when you input a new manual annotation file)")
    parser.add_argument("-g", "--gff",
                        help="GFF input file (tabulated)")
    parser.add_argument("-d", "--out-dir",
                        help="Output directory")
    args = parser.parse_args()

    # WARNING for shell use case:  
    # The script doesn't check if the inout files are valid. 
    # Files must be put in this order: $1=orthofinder file, $2=manual annotation file, $3=GFF file, $4=optional output directory)

    if not args.manual_annotation:
        logging.info("Please input a manual annotation file or a pickled manual annotation dictionary")
        sys.exit()
    if not args.orthofinder:
        logging.info("Please input an orthofinder output file")
        sys.exit()
    if not args.gff:
        logging.info("Please input a gff file")
        sys.exit()
    if not args.out_dir:
        args.out = "."

    orthologs_transfer(manual_annotation_file=args.manual_annotation,
                       orthofinder_file=args.orthofinder,
                       gff_file=args.gff,
                       outdir=args.out_dir)
